import json
import sys
import yaml

from .common import (
        get_host_info,
        get_build_number,
        )

def get_smoke_data(result):
    data = {}
    data['build_no'] = get_build_number()

    return data

def get_kernel_sru_data(result):
    data = {}

    return data

# XXX: this logic should probably be moved into the __init__ method
# of the result object since there will be one result object per
# master.run
def create_payload(result):
    publish_type = result.publish_type

    if publish_type is None:
        return None

    type_map = {
            'smoke': get_smoke_data,
            'kernel-sru': get_kernel_sru_data,
            }

    if publish_type and publish_type in type_map.iterkeys():
        return type_map[publish_type](result)

class Result(object):
    """
    Result collection class.
    """
    def __init__(self, name=None, testsuite=None, testcase=None,
            runlist=None, publish_type=None):
        self.results = []
        self.status = 'PASS'
        self.name = name
        self.testsuite = testsuite
        self.testcase = testcase
        self.runlist = runlist
        self.publish_type = publish_type

        self.errors = 0
        self.fetch_errors = 0
        self.failures = 0
        self.passes = 0

    def add_result(self, result):
        """
        Add a result to the object.

        Note: 'result' is expected to be a dictionary like this::

            {
                'command': '',
                'returncode': 0,
                'stdout': '',
                'stderr': '',
                'start_time': '',
                'time_delta': '',
            }
        """
        if result is None:
            return

        if self.testsuite is not None:
            result['testsuite'] = self.testsuite

        if self.testcase is not None:
            result['testcase'] = self.testcase

        self.results.append(result)

        if result['returncode'] == 1 and self.status != 'ERROR':
            self.status = 'FAIL'

        if result['returncode'] not in [0, 1]:
            self.status = 'ERROR'

    def result(self, verbose=False):
        """
        Output a text based result.
        """
        status = self.status
        sep = '-' * 70

        print sep

        if self.name:
            print self.name
            print sep

        for result in self.results:
            print "command: %s" % result['command']
            print "returned: %d" % result['returncode']
            print "started: %s" % result['start_time']
            print "runtime: %s" % result['time_delta']

            if self.testsuite is not None:
                print "testsuite: %s" % self.testsuite

            if self.testcase is not None:
                print "testcase: %s" % self.testcase

            if self.runlist is not None:
                print "runlist: %s" % self.runlist

            if self.status != 'PASS' or verbose and result['stdout'] != '':
                print "stdout: "
                print result['stdout']

            if self.status != 'PASS' or verbose and result['stderr'] != '':
                print "stderr: "
                print result['stderr']

        print

        data = self.payload()

        for key, value in data.iteritems():
            print '{}: {}'.format(key, value)

        self.clear_results()

        return status

    def clear_results(self):
        # reset the list of results so this object can be reused.
        self.results = []

        # reset the status, this should be safe since the only places that
        # should be calling clear_results() is testcase.run(), testsuite.run(),
        # and runner after attempting to fetch the testsuite.
        self.status = 'PASS'

    def count_results(self):
        return len(self.results)

    def publish_results(self):
        url = self.publish_url
        token = self.publish_token

        create_payload(self)

    def payload(self):
        """
        Construct the result payload.
        """
        host_info = get_host_info()
        data = {
                'runlist': self.runlist,
                'commands': self.results,
                'errors': self.errors,
                'failures': self.failures,
                'fetch_errors': self.fetch_errors,
                'passes': self.passes,
                'uname': list(host_info['uname']),
                'media-info': host_info['media-info'],
        }

        return data

class ResultYAML(Result):
    def result(self, verbose=False):

        if self.results:
            data = self.payload()
            yaml.dump(data, sys.stdout, explicit_start='---',
                      default_flow_style=False)

        status = self.status
        self.clear_results()

        return status

class ResultJSON(Result):
    def result(self, verbose=False):

        if self.results:
            data = self.payload()
            json.dump(data, sys.stdout, indent=4)

        status = self.status
        self.clear_results()

        return status
