import jsonschema
import os
import unittest
import yaml

from utah.client.runner import Runner

yaml_content = """---
- name: testsuite1
  fetch_cmd: get it now
- name: testsuite2
  fetch_cmd: fetch
"""

yaml_content_new = """---
timeout: 101
repeat_count: 99
testsuites:
    - name: testsuite1
      fetch_cmd: fetch1
    - name: testsuite2
      fetch_cmd: fetch2
"""

yaml_content_bad = [
"""---
testsuties:
    - name: testsuite1
      fetch_cmd: fetch1
""",  # typo
"""---
timeout: 101
repeat_count: 99
""",  # missing testsuites
"""---
- name: blah
- name: blip
""",  # missing fetch_cmds
"""---
testsuites:
  - name: blah
  - name: blip
""",  # missing fetch_cmds
]


class TestJSONSchema(unittest.TestCase):

    def test_orig_schema(self):
        """
        Test that the original master.run is correctly validated by the schema.
        """

        data = yaml.load(yaml_content)
        print("data: {}".format(data))
        jsonschema.validate(data, Runner.MASTER_RUNLIST_SCHEMA)

    def test_multi_schema(self):
        """
        Test that the new master.run is correctly validated by the schema.
        """

        data_new = yaml.load(yaml_content_new)
        print("data_new: {}".format(data_new))
        jsonschema.validate(data_new, Runner.MASTER_RUNLIST_SCHEMA)

    def test_example_runlist(self):
        example_runlist = os.path.join(os.path.dirname(__file__),
                                       "../examples/master.run")

        self.assertTrue(os.path.exists(example_runlist))

        with open(example_runlist, "r") as fp:
            data = yaml.load(fp)
            print("data: {}".format(data))
            jsonschema.validate(data, Runner.MASTER_RUNLIST_SCHEMA)
            fp.close()

    def test_bad_schemas(self):
        """
        Test that required fields are enforced by the schema.
        """

        for content in yaml_content_bad:
            data = yaml.load(content)
            print("data: {}".format(data))
            self.assertRaises(jsonschema.ValidationError,
                              jsonschema.validate,
                              data, Runner.MASTER_RUNLIST_SCHEMA)
