"""
Consolidate the routines used to run tests into one file.
"""
import logging
import os
import time
import urllib

from utah import config
from utah.exceptions import UTAHException


def run_tests(args, machine):
    # Since we're running from a script, output INFO
    for logger in [machine.logger] + machine.logger.handlers:
        if logger.level > logging.INFO:
            logger.setLevel(logging.INFO)

    if args.json:
        extraopts = ' -f json'
        suffix = '.json'
    else:
        extraopts = ''
        suffix = '.yaml'

    locallogs = []
    exitstatus = 0

    machine.installclient()
    for runlist in args.runlists:
        locallist = urllib.urlretrieve(runlist)[0]
        listname = os.path.basename(runlist)
        remotelog = os.path.join(config.logpath,
                machine.name + '_' + listname + '_tmp')
        remotelog = os.path.normpath(remotelog)
        locallog = os.path.join(config.logpath,
                machine.name + '_' + listname
                + time.strftime('_%Y-%m-%d_%H-%m-%S', time.gmtime())
                + suffix)
        locallog = os.path.normpath(locallog)
        try:
            machine.uploadfiles([locallist], os.path.normpath('/tmp'))
            options = (' -r /tmp/' + os.path.basename(locallist)
                       + ' -o ' + remotelog)
            utah_command = 'utah' + extraopts + options
            try:
                returncode = machine.run(utah_command, root=True)[0]
                if returncode != 0:
                    machine.logger.error('utah failed with return code: {}\n'
                                         .format(returncode))
                    exitstatus = 1
                    break
            except UTAHException as error:
                machine.logger.error('Failed to run test: ' + str(error))
                exitstatus = 1

            try:
                machine.downloadfiles([remotelog], locallog)
            except IOError as error:
                machine.logger.error('Failed to download results: '
                                     + str(error))
                exitstatus = 1
            else:
                machine.logger.error('Test log copied to ' + locallog)
                locallogs.append(locallog)
        finally:
            try:
                machine.run('rm -f ' + remotelog)
            except UTAHException as error:
                machine.logger.error('Failed to cleanup test: ' + str(error))
        if exitstatus != 0:
            break

    return exitstatus, locallogs
